// Cube.cpp: implementation of the CCube class.
//
//////////////////////////////////////////////////////////////////////
#define D3D_OVERLOADS
#include "d3d.h"
#include "Cube.h"

const int CUBE_VERTEX_COUNT = (4*6);
const int CUBE_INDEX_COUNT  = (6*6);

WORD cube_index[CUBE_INDEX_COUNT] =	
{	
	   // Set up the indices for the cube
     0+0,  0+1,  0+2,
     0+2,  0+1,  0+3,
     4+0,  4+1,  4+2,
     4+2,  4+1,  4+3,
     8+0,  8+1,  8+2,
     8+2,  8+1,  8+3,
     12+0, 12+1, 12+2,
     12+2, 12+1, 12+3,
     16+0, 16+1, 16+2,
     16+2, 16+1, 16+3,
     20+0, 20+1, 20+2,
     20+2, 20+1, 20+3,
};
	

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCube::CCube(D3DVECTOR origin,D3DVECTOR size,float R,float G,float B)
{
	// Zero out the material information
	ZeroMemory( &material, sizeof(material) );
	
	// allocate the vertex array

	verts=new D3DVERTEX[CUBE_VERTEX_COUNT];

	// calculate far corner of the cube

	D3DVECTOR extent=origin+size;

	/* old vertices : old position information is now vec0-vec7
					  uv information is now u0-u7 and v0-v7 respectively
					  light information has been removed and replaced with normals per vertex

	verts[0]=D3DVERTEX(D3DVECTOR(origin.x,origin.y,origin.z), normal0, 0.0f,0.0f);
	verts[1]=D3DVERTEX(D3DVECTOR(origin.x,extent.y,origin.z), normal0, 0.0f,2.0f);
	verts[2]=D3DVERTEX(D3DVECTOR(extent.x,origin.y,origin.z), normal0, 1.0f,0.0f);
	verts[3]=D3DVERTEX(D3DVECTOR(extent.x,extent.y,origin.z), normal0, 1.0f,2.0f);
	verts[4]=D3DVERTEX(D3DVECTOR(extent.x,origin.y,extent.z), normal0, 2.0f,0.0f);
	verts[5]=D3DVERTEX(D3DVECTOR(extent.x,extent.y,extent.z), normal0, 2.0f,2.0f);
	verts[6]=D3DVERTEX(D3DVECTOR(origin.x,origin.y,extent.z), normal0, 1.0f,0.0f);
	verts[7]=D3DVERTEX(D3DVECTOR(origin.x,extent.y,extent.z), normal0, 1.0f,2.0f);
	*/

	// Original 3d points
	D3DVECTOR vec0(origin.x,origin.y,origin.z);
	D3DVECTOR vec1(origin.x,extent.y,origin.z);
	D3DVECTOR vec2(extent.x,origin.y,origin.z);
	D3DVECTOR vec3(extent.x,extent.y,origin.z);
	D3DVECTOR vec4(extent.x,origin.y,extent.z);
	D3DVECTOR vec5(extent.x,extent.y,extent.z);
	D3DVECTOR vec6(origin.x,origin.y,extent.z);
	D3DVECTOR vec7(origin.x,extent.y,extent.z);

	// Define the normals for the cube
    D3DVECTOR normal0( 0.0f, 0.0f,-1.0f ); // Front face
    D3DVECTOR normal1( 1.0f, 0.0f, 0.0f ); // Right face
    D3DVECTOR normal2( 0.0f, 0.0f, 1.0f ); // Back face
    D3DVECTOR normal3(-1.0f, 0.0f, 0.0f ); // Left face
    D3DVECTOR normal4( 0.0f, 1.0f, 0.0f ); // Top face
    D3DVECTOR normal5( 0.0f,-1.0f, 0.0f ); // Bottom face

	D3DVALUE u0,u1,u2,u3,u4,u5,u6,u7;
	D3DVALUE v0,v1,v2,v3,v4,v5,v6,v7;

	u0 = 0.0f; v0 = 0.0f;
	u1 = 0.0f; v1 = 2.0f;
	u2 = 1.0f; v2 = 0.0f;
	u3 = 1.0f; v3 = 2.0f;
	u4 = 2.0f; v4 = 0.0f;
	u5 = 2.0f; v5 = 2.0f;
	u6 = 1.0f; v6 = 0.0f;
	u7 = 1.0f; v7 = 2.0f;

	// Set up the 8 corners of the cube, this time setting up
	//  vertex normals as well.
	//	Note: First vector is position.	Second vector is normal.
	
	// front face
	verts[0]=D3DVERTEX( vec0, normal0, u0, v0 );
	verts[1]=D3DVERTEX( vec1, normal0, u1, v1 );
	verts[2]=D3DVERTEX( vec2, normal0, u2, v2 );
	verts[3]=D3DVERTEX( vec3, normal0, u3, v3 );

	// right
	verts[4]=D3DVERTEX( vec2, normal1, u2, v2 );
	verts[5]=D3DVERTEX( vec3, normal1, u3, v3 );
	verts[6]=D3DVERTEX( vec4, normal1, u4, v4 );
	verts[7]=D3DVERTEX( vec5, normal1, u5, v5 );


	//  back face
	verts[ 8]=D3DVERTEX( vec4, normal2, u4, v4 );
	verts[ 9]=D3DVERTEX( vec5, normal2, u5, v5 );
	verts[10]=D3DVERTEX( vec6, normal2, u6, v6 );
	verts[11]=D3DVERTEX( vec7, normal2, u7, v7 );

	// left face
	verts[12]=D3DVERTEX( vec6, normal3, u6, v6 );
	verts[13]=D3DVERTEX( vec7, normal3, u7, v7 );
	verts[14]=D3DVERTEX( vec0, normal3, u0, v0 );
	verts[15]=D3DVERTEX( vec1, normal3, u1, v1 );
	
	//  top face
	verts[16]=D3DVERTEX( vec1, normal4, u1, v1 );
	verts[17]=D3DVERTEX( vec7, normal4, u7, v7 );
	verts[18]=D3DVERTEX( vec3, normal4, u3, v3 );
	verts[19]=D3DVERTEX( vec5, normal4, u5, v5 );

	// bottom face
	verts[20]=D3DVERTEX( vec6, normal5, u6, v6 );
	verts[21]=D3DVERTEX( vec0, normal5, u0, v0 );
	verts[22]=D3DVERTEX( vec4, normal5, u4, v4 );
	verts[23]=D3DVERTEX( vec2, normal5, u2, v2 );

}


#define SafeDelete(x) if (x) {delete x;x=NULL;}

CCube::~CCube()
{
	// de-allocate the vertex array

	SafeDelete(verts);
}

void CCube::draw(LPDIRECT3DDEVICE7 device)
{
	// Set RenderState to reflect the cube's material 
	device->SetMaterial( &material );
	
	// Draw
	device->DrawIndexedPrimitive(D3DPT_TRIANGLELIST,D3DFVF_VERTEX,
								 verts,CUBE_VERTEX_COUNT,
								 cube_index,CUBE_INDEX_COUNT,
								 0);
}




////////////////////////////////////////////////////////////////////////////////
//		The following 3 functions set the material properties for the cube.
//	All of the information is contained in one D3DMATERIAL7 structure. It is
//	split up into functions by the materials components. Do not forget to initilize
//	the material to zero (as in constructor) in derived works, otherwise you may 
//	get unreliable results.
//		Also note that materials can be set on a per primitive (ie triangle, strip)
//	basis.  A cube could have a highly specular side, and a dull, non-specular side.
//	
void CCube::SetMaterialDiffuse(  float diffuseR,  float diffuseG,  float diffuseB,  float diffuseA  )
{
	// Set the RGBA for diffuse reflection
	material.dcvDiffuse.r = (D3DVALUE) diffuseR; 
    material.dcvDiffuse.g = (D3DVALUE) diffuseG; 
    material.dcvDiffuse.b = (D3DVALUE) diffuseB; 
    material.dcvDiffuse.a = (D3DVALUE) diffuseA; 
}

void CCube::SetMaterialAmbient(  float ambientR,  float ambientG,  float ambientB,  float ambientA  )
{
   // Set the RGBA for ambient reflection. 
    material.dcvAmbient.r = (D3DVALUE)  ambientR;
    material.dcvAmbient.g = (D3DVALUE)  ambientB; 
    material.dcvAmbient.b = (D3DVALUE)  ambientG; 
    material.dcvAmbient.a = (D3DVALUE)  ambientA; 
 
}

void CCube::SetMaterialSpecular( float specularR, float specularG, float specularB, float specularA, float specularPower )
{
    // Set the RGBA and the sharpness of the specular highlight
    material.dcvSpecular.r = (D3DVALUE) specularR; 
    material.dcvSpecular.g = (D3DVALUE) specularG; 
    material.dcvSpecular.b = (D3DVALUE) specularB; 
    material.dcvSpecular.a = (D3DVALUE) specularA; 
    material.dvPower = (float) specularPower;

}
// End material section
////////////////////////////////////////////////////////////////////////////////

 
